#ifndef  CLIB_GAMES_PROTOS_H
#define  CLIB_GAMES_PROTOS_H

/*
**	$VER: games_protos.h 0.02 (05.7.96)
**	Includes Release xx.xx
**
**	C prototypes.
**
**	(C) Copyright 1996 DreamWorld Productions.
**	    All Rights Reserved
*/

#ifndef  GAMES_GMS_H
#include <games/games.h>
#endif
#ifndef  EXEC_TYPES_H
#include <exec/types.h>
#endif
#ifndef  EXEC_TASKS_H
#include <exec/tasks.h>
#endif

/* Standard functions */

ULONG InitGPI(UWORD GPI_ID);
ULONG RemoveGPI(UWORD GPI_ID);

ULONG Read_Mouse(UWORD PortName);
ULONG Read_JoyPort(UWORD Portname);
ULONG Read_JoyStick(UWORD PortName);
ULONG Read_JoyPad(UWORD PortName);
ULONG Read_SegaPad(UWORD PortName);
ULONG Read_Analog(UWORD PortName);
ULONG Read_Key(struct KeyStruct *);

ULONG FastRandom(UWORD Range);
ULONG SlowRandom(UWORD Range);
void  Wait_LMB(void);
void  Wait_Fire(UWORD PortName);
void  Wait_Time(UWORD MicroSeconds);
void  NoRequesters(void);
void  SetFilter(UWORD NewStatus);

void  Add_InputHandler(void);
void  Rem_InputHandler(void);
void  Add_Interrupt(APTR Interrupt, UWORD IntNum, UWORD IntPri);
void  Rem_Interrupt(APTR IntBase);

APTR  SmartLoad(APTR FileName, APTR Destination, ULONG MemType);
APTR  QuickLoad(APTR FileName, APTR Destination, ULONG MemType);
APTR  SmartUnpack(APTR Source, APTR Destination, ULONG MemType);
void  SetUserPri(void);
void  SetGMSPrefs(struct PrefsStruct *);
void  UnpackPic(APTR PicSource, APTR PicDest, APTR PaletteDest);
APTR  GetPicInfo(APTR Picture);
void  SetPassword(ULONG Password);
APTR  AllocMemBlock(ULONG Size, ULONG MemType);
void  FreeMemBlock(APTR MemBlock);

/* Screens.GPI functions */

ULONG Add_Screen(struct GameScreen *);
void  Delete_Screen(struct GameScreen *);
void  Show_Screen(struct GameScreen *);
void  Hide_Screen(struct GameScreen *);
void  ReturnToOS(void);
void  AutoOSReturn(void);
void  SwapBuffers(struct GameScreen *);
void  Wait_VBL(void);
void  Wait_OSVBL(void);
void  Wait_RastLine(UWORD LinePosition);
void  Remake_Screen(struct GameScreen *);
void  HWScroll_Horizontal(struct GameScreen *, UWORD XShift);
void  HWScroll_Vertical(struct GameScreen *, UWORD YShift);
void  HWScroll_Reset(struct GameScreen *);

/* Palette stuff */

UWORD B12_FadeToBlack(struct GameScreen *);
UWORD B12_FadeToWhite(struct GameScreen *, UWORD StartCol, UWORD AmtCols);
UWORD B12_FadeToPalette(struct GameScreen *, APTR Palette, UWORD StartCol, UWORD AmtCols);
UWORD B12_FadeToColour(struct GameScreen *, UWORD RGB);
UWORD B24_FadeToBlack(struct GameScreen *, UWORD Speed);
UWORD B24_FadeToWhite(struct GameScreen *);
UWORD B24_FadeToPalette(struct GameScreen *, APTR Palette);
UWORD B24_FadeToColour(struct GameScreen *, ULONG RRGGBB);
void  Change_Colours(struct GameScreen *, APTR Colours, UWORD StartColour, UWORD AmtColours);
void  Blank_Colours(struct GameScreen *);

/* Rasterlist stuff. */

ULONG Init_RasterList(struct GameScreen *);
void  Update_RasterList(struct GameScreen *);
void  Remove_RasterList(struct GameScreen *);
void  Hide_RasterList(struct GameScreen *);
void  Show_RasterList(struct GameScreen *);

/* Sprite stuff. */

void  Init_Sprite(struct SpriteStruct *);
void  Update_Sprite(struct GameScreen *, struct SpriteStruct *);
void  Move_Sprite(struct GameScreen *);
void  Remove_Sprite(struct GameScreen *, UWORD SpriteNum);
void  Update_SpriteList(struct SpriteList *);
void  Remove_SpriteList(struct SpriteList *);
void  Remove_AllSprites(void);
void  Return_AllSprites(void);

/* Blitter.GPI functions.  NOT IMPLEMENTED YET. */

void  Init_BOB(struct GameScreen *, struct BOBStruct);
void  Init_BOBList(struct GameScreen *, APTR BOBList);
void  Blit_BOB(struct GameScreen *, struct BOBStruct);
void  Blit_BOBList(struct GameScreen *, APTR BOBList);
void  Clear_BOB(struct GameScreen *, struct BOBStruct);
void  Clear_BOBList(struct GameScreen *, APTR BOBList);
void  Blit_Vertical_Series(struct GameScreen *, struct SeriesStruct *);
void  Blit_ScaledBMap(struct GameScreen *);
void  Blit_Pattern(struct GameScreen *);
void  Blit_Line(struct GameScreen *, UWORD SX, UWORD SY, UWORD EX, UWORD EY);
void  Blit_Direct(struct GameScreen *, struct DirectValues *);
void  SnR_Backgrounds(struct GameScreen *, APTR BOBList);
void  Clear_Screen(struct GameScreen *);
void  Clear_ScreenArea(struct GameScreen *, UWORD TopX, UWORD TopY, UWORD BotX, UWORD BotY);
void  GrabBlitter(void);
void  FreeBlitter(void);

/* Sound.GPI functions. */

UWORD AllocAudio(void);
void  FreeAudio(void);
void  InitSound(struct SoundStruct *);
UWORD CheckChannel(UWORD ChannelName);
void  PlaySound(struct SoundStruct *);
void  PlaySoundDAC1(struct SoundStruct *);
void  PlaySoundDAC2(struct SoundStruct *);
void  PlaySoundDAC3(struct SoundStruct *);
void  PlaySoundDAC4(struct SoundStruct *);
void  PlaySoundPri(struct SoundStruct *);
void  PlaySoundPriDAC1(struct SoundStruct *);
void  PlaySoundPriDAC2(struct SoundStruct *);
void  PlaySoundPriDAC3(struct SoundStruct *);
void  PlaySoundPriDAC4(struct SoundStruct *);

#endif /* CLIB_GAMES_PROTOS_H */
